@@ -10,6 +10,7 @@ import numpy as np
 import matplotlib.pyplot as plt
 import torch
 import torchvision
+import cv2
 
 from mohou.constant import N_DATA_INTACT
 from mohou.file import load_object, dump_object
@@ -118,6 +119,10 @@ class ImageBase(ElementBase):
     def to_rgb(self, *args, **kwargs) -> 'RGBImage':
         pass
 
+    @abstractmethod
+    def resize(self, shape2d_new: Tuple[int, int]) -> None:
+        pass
+
 
 class PrimitiveImageBase(PrimitiveElementBase, ImageBase):
     _channel: ClassVar[int]
@@ -162,6 +167,9 @@ class RGBImage(PrimitiveImageBase):
     def to_rgb(self, *args, **kwargs) -> 'RGBImage':
         return self
 
+    def resize(self, shape2d_new: Tuple[int, int]) -> None:
+        self._data = cv2.resize(self._data, shape2d_new, interpolation=cv2.INTER_AREA)
+
 
 class DepthImage(PrimitiveImageBase):
     _channel: ClassVar[int] = 1
@@ -203,13 +211,16 @@ class DepthImage(PrimitiveImageBase):
         plt.close(fig)
         return RGBImage(arr)
 
+    def resize(self, shape2d_new: Tuple[int, int]) -> None:
+        tmp = cv2.resize(self._data, shape2d_new, interpolation=cv2.INTER_CUBIC)
+        self._data = np.expand_dims(tmp, axis=2)
+
 
 class CompositeImageBase(ImageBase):
     image_types: ClassVar[List[Type[PrimitiveImageBase]]]
     images: List[PrimitiveImageBase]
-    _shape: Tuple[int, int, int]
 
-    def __init__(self, images: List[PrimitiveImageBase], check_size=False):
+    def __init__(self, images: List[PrimitiveImageBase], check_size=True):
 
         image_shape = images[0].shape[:2]
 
@@ -217,15 +228,18 @@ class CompositeImageBase(ImageBase):
             for image in images:
                 assert_with_message(image.shape[:2], image_shape, 'image w-h')
 
+        for image, image_type in zip(images, self.image_types):
+            assert_isinstance_with_message(image, image_type)
+
         self.images = images
-        self._shape = (image_shape[0], image_shape[1], self.channel())
 
     def to_tensor(self) -> torch.Tensor:
         return torch.cat([im.to_tensor() for im in self.images])
 
     @property
     def shape(self) -> Tuple[int, int, int]:
-        return self._shape
+        shape_2d = self.images[0].shape[:2]
+        return (shape_2d[0], shape_2d[1], self.channel())
 
     def randomize(self: CompositeImageT) -> CompositeImageT:
         rand = copy.deepcopy(self)
@@ -257,6 +271,10 @@ class CompositeImageBase(ImageBase):
                 return image
         assert False
 
+    def resize(self, shape2d_new: Tuple[int, int]) -> None:
+        for image in self.images:
+            image.resize(shape2d_new)
+
 
 class RGBDImage(CompositeImageBase):
     image_types = [RGBImage, DepthImage]
@@ -278,14 +296,20 @@ class ElementDict(OrderedDict[Type[ElementBase], ElementBase]):
     def __getitem__(self, key: Type[ElementT]) -> ElementT:
         if issubclass(key, PrimitiveElementBase):
             return super().__getitem__(key)  # type: ignore
+
         elif issubclass(key, CompositeImageBase):
-            images = []
+            if key in self:
+                return super().__getitem__(key)  # type: ignore
+
             # TODO(HiroIshida) somehow, if the following is written in comprehension
             # then we get "TypeError: super(type, obj): obj must be an instance or subtype of type"
+            images = []
             for imt in key.image_types:
                 images.append(super().__getitem__(imt))
             return key(images)  # type: ignore
-        assert False
+
+        else:
+            assert False
 
 
 def get_all_concrete_types() -> List[Type[ElementBase]]:
@@ -329,7 +353,7 @@ def create_composite_image_sequence(composite_image_type: Type[CompositeImageT],
 
 class EpisodeData:
     type_shape_table: OrderedDict[Type[ElementBase], Tuple[int, ...]]
-    sequence_list: Tuple[ElementSequence, ...]
+    sequence_tuple: Tuple[ElementSequence, ...]
 
     def __init__(self, sequence_tuple: Tuple[ElementSequence, ...]):
         for sequence in sequence_tuple:
@@ -347,10 +371,10 @@ class EpisodeData:
         assert all_different_type, 'all sequences must have different type'
 
         self.type_shape_table = type_shape_table
-        self.sequence_list = sequence_tuple
+        self.sequence_tuple = sequence_tuple
 
     def filter_by_primitive_type(self, elem_type: Type[PrimitiveElementT]) -> ElementSequence[PrimitiveElementT]:
-        for seq in self.sequence_list:
+        for seq in self.sequence_tuple:
             if isinstance(seq[0], elem_type):
                 # thanks to all_different_type
                 return seq
@@ -367,7 +391,7 @@ class EpisodeData:
             assert False
 
     def __iter__(self):
-        return self.sequence_list.__iter__()
+        return self.sequence_tuple.__iter__()
 
 
 class MultiEpisodeChunk:
